//-----------------------------------------------------------------------
// <copyright file="FileListView.cs" company="Kanazawa-soft-design,LLC.">
//     Copylight(C) 2018 Kanazawa-soft-design,LLC.
// </copyright>
// <author>金澤 宣明</author>
// <summary></summary>
//-----------------------------------------------------------------------

using System;
using System.Drawing;
using System.Windows.Forms;
using System.IO;

namespace WinAppCS001
{
	public partial class MainForm
	{
		// -----------------------------------------------------------------------
		/// <summary>
		/// ファイルデータ表示ビューを初期化する.
		/// </summary>
		private void FileData_Init()
		{
			//! 垂直スクロールバーを初期化する.
			FileDataVScroll.Value   = 0;
			FileDataVScroll.Minimum = 0;
			FileDataVScroll.Maximum = 0;

			//! 水平スクロールバーを初期化する.
			FileDataHScroll.Value   = 0;
			FileDataHScroll.Minimum = 0;
			FileDataHScroll.Maximum = 0;

			//! ホイールイベントの追加.
			FileData.MouseWheel += new System.Windows.Forms.MouseEventHandler( FileData_MouseWheel );
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// ファイルを読み込む.
		/// </summary>
		/// <param name="path">
		/// 読み込むファイル名.
		/// </param>
		/// <returns>
		/// 読み込んだファイル･データを格納したバイト配列.
		/// </returns>
		private byte[] ReadFile( string path )
		{
			//! ファイルを読み込みモードでオープンする.
			using( var fs = new FileStream( path, FileMode.Open, FileAccess.Read ) ){

				//! ファイルのバイト数を取得する.
				var size = fs.Length;

				/*!
				 * ファイルサイズが読み込みサイズの上限を超える場合は
				 * サイズの上限までしか読み込まない.
				 */
				if( size > MaxFileSize ){
					size = MaxFileSize;
				}
				//! ファイルサイズ分のファイルの読み込み用バッファを確保する.
				var buf = new byte[size];

				//! ファイルを読み込み用バッファに1回で読み込む.
				fs.Read( buf, 0, buf.Length );
				return buf;
			}
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// ファイルの内容を表示する.
		/// </summary>
		private void ShowFile()
		{
			//! 垂直スクロールバーを初期化する.
			FileDataVScroll.Value   = 0;
			FileDataVScroll.Minimum = 0;
			FileDataVScroll.Maximum = (FileBuff.Length / 16);

			//! 水平スクロールバーを初期化する.
			FileDataHScroll.Value   = 0;
			FileDataHScroll.Minimum = 0;
			FileDataHScroll.Maximum = 74;

			//! 画面に合わせてスクロールバーのつまみ幅を設定する.
			SetScrollBar();

			//! バイナリ表示画面を再描画する.
			splitContainer2.Panel2.Refresh();
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// ファイルデータ表示ビューの描画イベント処理.
		/// </summary>
		/// <param name="sender">
		/// イベントを発生させたオブジェクト.
		/// </param>
		/// <param name="e">
		/// イベント･パラメータ.
		/// </param>
		private void FileData_Paint( object sender, PaintEventArgs e )
		{
			var rc = e.ClipRectangle;
			var g  = e.Graphics;

			//! 画面をクリアする.
			g.FillRectangle( Brushes.White, 0, 0, rc.Width, rc.Height );

			//! ファイルデータが無い場合は描画を行わない.
			if( FileBuff == null ){
				return;
			}
			//! フォントを作成する.
			Font fnt = new Font( "ＭＳ ゴシック", 12 );

			float x = 0;
			float y = 0;

			int colomuns = 1;
			using( StringFormat sf = new StringFormat( StringFormat.GenericTypographic ) ){

				//! 1文字の幅を取得する.
				var fs = g.MeasureString( "0", fnt, rc.Width, sf );

				//! 水平方向の書き出し座標を計算する.
				x = -1 * (FileDataHScroll.Value * fs.Width);

				//! 1画面に表示可能な文字数を計算する.
				if( fs.Width > 0 ){
					colomuns = (int)(rc.Width / fs.Width);
				}
			}

			//! 1画面に表示可能な行数を計算する.
			int lines = (fnt.Height > 0) ? ((rc.Height) + (fnt.Height - 1)) / fnt.Height : 1;

			//! スケールを描画する.
			y = DrawScale( g, rc, fnt, x, y );

			//! ファイルデータを16進数で描画する.
			int adr = FileDataVScroll.Value * 16;
			for( int i = 1; i <= lines; i++ ){
				DrawBinaryData( g, rc, fnt, x, y, adr );
				adr += 16;
				y   += fnt.Height;
			}
			//! フォントを破棄する.
			fnt.Dispose();
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// スケールを描画する.
		/// </summary>
		/// <param name="g">
		/// 描画に使用するグラフィックス.
		/// </param>
		/// <param name="rc">
		/// クライアント領域のサイズ.
		/// </param>
		/// <param name="fnt">
		/// フォント.
		/// </param>
		/// <param name="x">
		/// 描画開始するX座標.
		/// </param>
		/// <param name="y">
		/// 描画開始するY座標.
		/// </param>
		/// <returns>
		/// 次に描画するY座標.
		/// </returns>
		private float DrawScale( Graphics g, Rectangle rc, Font fnt, float x, float y )
		{
			using( StringFormat sf = new StringFormat( StringFormat.GenericTypographic ) ){

				//! アドレスを描画する領域を塗りつぶす.
				var adrArea = g.MeasureString( "000000000", fnt, rc.Width, sf );
				g.FillRectangle( Brushes.Gray, x, 0, adrArea.Width, rc.Height );

				//! スケールを描画する.
				g.FillRectangle( Brushes.Gray, 0, 0, rc.Width, fnt.Height );
				g.DrawString( "ADDRESS:  +0 +1 +2 +3 +4 +5 +6 +7 +8 +9 +A +B +C +D +E +F ASCII", fnt, Brushes.White, x, y, sf );
				y += fnt.Height;
			}
			return y;
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// 1行分のバイナリ･データを描画する.
		/// </summary>
		/// <param name="g">
		/// 描画に使用するグラフィックス.
		/// </param>
		/// <param name="rc">
		/// クライアント領域のサイズ.
		/// </param>
		/// <param name="fnt">
		/// フォント.
		/// </param>
		/// <param name="x">
		/// 描画開始するX座標.
		/// </param>
		/// <param name="y">
		/// 描画開始するY座標.
		/// </param>
		/// <param name="adr">
		/// 描画開始するバイナリデータのアドレス.
		/// </param>
		private void DrawBinaryData( Graphics g, Rectangle rc, Font fnt, float x, float y, int adr )
		{
			if( adr >= FileBuff.Length ){
				return;
			}
			using( StringFormat sf = new StringFormat( StringFormat.GenericTypographic ) ){

				var fs = g.MeasureString( "0", fnt, rc.Width, sf );

				//! アドレスを描画する.
				string adrText = adr.ToString( "X8" );
				g.DrawString( adrText, fnt, Brushes.White, x, y, sf );

				var x1 = x + (9 * fs.Width);

				string ascii = string.Empty;
				for( int i = 0; i < 16; i++ ){
					string byteText = string.Empty;
					if( (adr + i) < FileBuff.Length ){
						uint c = FileBuff[adr + i];

						byteText = " " + c.ToString( "X2" );

						ascii += ((0x20 <= c && c < 0x80) ? (char)c : '.');
					}else{
						byteText = "   ";
					}
					g.DrawString( byteText, fnt, Brushes.Black, x1, y, sf );
					x1 += (3 * fs.Width);
				}
				string asciiText = " " + ascii;
				g.DrawString( asciiText, fnt, Brushes.Black, x1, y, sf );
			}
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// スクロールバーを設定する.
		/// </summary>
		private void SetScrollBar()
		{
			//! クライアント領域のサイズを取得する.
			System.Drawing.Size rc = FileData.ClientSize;

			//! グラフィックス･オブジェクトを取得する.
			Graphics g = FileData.CreateGraphics();

			//! フォントを作成する.
			Font fnt = new Font( "ＭＳ ゴシック", 12 );

			//! 1画面に表示可能な行数を計算する.
			int lines = (fnt.Height > 0) ? ((rc.Height) + (fnt.Height - 1)) / fnt.Height : 1;

			//! 1画面に表示可能な文字数を計算する.
			int colomuns = 1;
			using( StringFormat sf = new StringFormat( StringFormat.GenericTypographic ) ){

				//! 1文字の幅を取得する.
				var fs = g.MeasureString( "0", fnt, rc.Width, sf );

				//! 1画面に表示可能な文字数を計算する.
				if( fs.Width > 0 ){
					colomuns = (int)(rc.Width / fs.Width);
				}
			}

			//! スクロールバーのつまみ幅を画面に合わせて再設定する.
			FileDataVScroll.LargeChange = lines    / 2;
			FileDataHScroll.LargeChange = colomuns / 2;

			//! フォントを破棄する.
			fnt.Dispose();

			//! グラフィックス･オブジェクトを解放する.
			g.Dispose();
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// バイナリ表示画面の垂直スクロールバーが操作されたときのイベント処理.
		/// </summary>
		/// <param name="sender">
		/// イベントを発生させたオブジェクト.
		/// </param>
		/// <param name="e">
		/// イベント･パラメータ.
		/// </param>
		private void FileDataVScroll_Scroll( object sender, ScrollEventArgs e )
		{
			//! バイナリ表示画面を再描画する.
			splitContainer2.Panel2.Refresh();
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// バイナリ表示画面の水平スクロールバーが操作されたときのイベント処理.
		/// </summary>
		/// <param name="sender">
		/// イベントを発生させたオブジェクト.
		/// </param>
		/// <param name="e">
		/// イベント･パラメータ.
		/// </param>
		private void FileDataHScroll_Scroll( object sender, ScrollEventArgs e )
		{
			//! バイナリ表示画面を再描画する.
			splitContainer2.Panel2.Refresh();
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// マウス･ホイールのイベント処理.
		/// </summary>
		/// <param name="sender">
		/// イベントを発生させたオブジェクト.
		/// </param>
		/// <param name="e">
		/// イベント･パラメータ.
		/// </param>
		private void FileData_MouseWheel( object sender, MouseEventArgs e )
		{
			//! スクロール量（方向）の表示.
			var val = FileDataVScroll.Value - (e.Delta * SystemInformation.MouseWheelScrollLines / 120);

			if( val < FileDataVScroll.Minimum ){
				val = FileDataVScroll.Minimum;
			}
			int maximun = FileDataVScroll.Maximum - (FileDataVScroll.LargeChange - 1);
			if( val > maximun ){
				val = maximun;
			}
			FileDataVScroll.Value = val;

			//! バイナリ表示画面を再描画する.
			splitContainer2.Panel2.Refresh();
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// ファイルデータ表示ビューがクリックされたときのイベント処理.
		/// </summary>
		/// <param name="sender">
		/// イベントを発生させたオブジェクト.
		/// </param>
		/// <param name="e">
		/// イベント･パラメータ.
		/// </param>
		private void FileData_Click( object sender, EventArgs e )
		{
			//! ピクチャボックスにフォーカスを移す.
			FileData.Focus();
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// ウインドウサイズが変更されたときのイベント処理.
		/// </summary>
		/// <param name="sender">
		/// イベントを発生させたオブジェクト.
		/// </param>
		/// <param name="e">
		/// イベント･パラメータ.
		/// </param>
		private void FileData_SizeChanged( object sender, EventArgs e )
		{
			//! スクロールバーを設定する.
			SetScrollBar();
		}
	}
}
