//-----------------------------------------------------------------------
// <copyright file="FileListView.cs" company="Kanazawa-soft-design,LLC.">
//     Copylight(C) 2018 Kanazawa-soft-design,LLC.
// </copyright>
// <author>金澤 宣明</author>
// <summary></summary>
//-----------------------------------------------------------------------

using System;
using System.Windows.Forms;
using System.IO;

namespace WinAppCS001
{
	public partial class MainForm
	{
		// -----------------------------------------------------------------------
		/// <summary>
		/// ファイル一覧のビューを初期化する.
		/// </summary>
		private void FileList_Init()
		{
			//! ファイル一覧にカラムを追加する.
			FileList.Columns.Add( "ファイル名", 200, HorizontalAlignment.Left  );
			FileList.Columns.Add( "サイズ"    , 120, HorizontalAlignment.Right );
			FileList.Columns.Add( "更新日時"  , 150, HorizontalAlignment.Left  );

			//! ファイル一覧にドライブ名を表示する.
			ShowDriveName();
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// ファイル一覧を更新する.
		/// </summary>
		/// <param name="path">
		/// 一覧表示するディレクトリ名.
		/// </param>
		private void UpdateFileList( string path )
		{
			//! 一覧表示するディレクトリ名を保存する.
			CurrentDir = path;

			if( CurrentDir.Length > 0 ){
				//! 指定されたパスのファイル一覧を生成する.
				ShowFileName( path );
			}else{
				//! パスが指定されていない場合はドライブ一覧を生成する.
				ShowDriveName();
			}
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// ファイル一覧にドライブ名の一覧を表示する.
		/// </summary>
		private void ShowDriveName()
		{
			//! ファイル一覧の表示を一旦クリアする.
			FileList.Items.Clear();

			//! 論理ドライブ名をすべて取得する.
			string[] stDrives = Directory.GetLogicalDrives();

			//! 取得した論理ドライブ名をすべて表示する.
			foreach( var drive in stDrives ){

				//! ドライブ名をの最後にあるセパレータ("\")を取り除く.
				var drvname = drive.Substring( 0, drive.Length - 1 );

				//! ファイル一覧に生成したファイル情報を追加する.
				FileList.Items.Add( new ListViewItem( new string[] { drvname, "", "" }) );
			}
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// ファイル一覧にファイル名の一覧を表示する.
		/// </summary>
		/// <param name="path">
		/// ファイル一覧を表示するディレクトリの絶対パス.
		/// </param>
		private void ShowFileName( string path )
		{
			//! ファイル一覧の表示を一旦クリアする.
			FileList.Items.Clear();

			//! 一つ上のディレクトリに移動する為の「..」をリストの先頭に追加する.
			FileList.Items.Add( new ListViewItem( new string[] { "..", "<DIR>", "" }) );

			try {
				//! 指定されたディレクトリ直下のサブディレクトリ名のリストを取得する.
				path += "\\";
				foreach( var dir in Directory.GetDirectories( path ) ){

					//! ディレクトリの属性を取得する.
					var di = new DirectoryInfo( dir );

					//! 隠しディレクトリはリストに表示しない.
					if( (di.Attributes & FileAttributes.Hidden) != FileAttributes.Hidden ){

						//! 更新日付を文字列にする.
						var dateStr = di.LastWriteTime.ToString( "yyyy/MM/dd hh:mm" );

						//! ファイル一覧に生成したディレクトリ情報を追加する.
						FileList.Items.Add( new ListViewItem( new string[] { di.Name, "<DIR>", dateStr }) );
					}
				}

				//! 指定されたディレクトリ直下のファイル名のリストを取得する.
				foreach( var file in Directory.GetFiles( path, "*", SearchOption.TopDirectoryOnly ) ){

					//! ファイル属性を取得する.
					var attr = File.GetAttributes( file );

					//! 隠しファイルはリストに表示しない.
					if( (attr & FileAttributes.Hidden) != FileAttributes.Hidden ){

						//! ファイル名からファイル情報を取得する.
						var fi = new FileInfo( file );

						//! ファイルサイズを右詰の文字列に変換する.
						var sizeStr = fi.Length.ToString( "#,0" );

						//! 更新日付を文字列にする.
						var dateStr = fi.LastWriteTime.ToString( "yyyy/MM/dd hh:mm" );

						//! ファイル一覧に生成したファイル情報を追加する.
						FileList.Items.Add( new ListViewItem( new string[] { fi.Name, sizeStr, dateStr }) );
					}
				}
			}catch( Exception ){
			}
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// ファイル一覧の項目がダブルクリックされたときの操作イベント.
		/// </summary>
		/// <param name="sender">
		/// イベントを発生させたオブジェクト.
		/// </param>
		/// <param name="e">
		/// イベント･パラメータ.
		/// </param>
		private void FileList_DoubleClick( object sender, EventArgs e )
		{
			if( FileList.SelectedItems.Count > 0 ){

				/*!
				 * リストビューからファイル名を取り出して、カレントディレクトリ名と連結して
				 * 選択されたファイルの絶対パスにする.
				 */
				string filename = FileList.SelectedItems[0].Text;
				string fullpath;
				if( CurrentDir.Length > 0 ){
					fullpath = CurrentDir + "\\" + filename;
				}else{
					fullpath = filename;
				}

				//! ".."の場合、カレントパスの最後のディレクトリ名を取り除く.
				if( filename.Equals( ".." ) ){
					int pos = CurrentDir.LastIndexOf( '\\' );
					if( pos >= 0 ){
						fullpath = CurrentDir.Substring( 0, pos );
					}else{
						ExpandDirectory( "" );
						return;
					}
				}

				var attr = File.GetAttributes( fullpath );
				if( (attr & FileAttributes.Directory) == FileAttributes.Directory ){
					//! 選択されたファイルがディレクトリの場合、選択されたディレクトリに移動する.
					ExpandDirectory( fullpath );
				}else{
					//! ファイルを読み込む.
					FileBuff = ReadFile( fullpath );

					//! ステータスバーにファイル名を表示する.
					StatusLabel1.Text = fullpath;

					//! 読み込んだファイルの内容を表示する.
					ShowFile();
				}
			}
		}
	}
}
