//-----------------------------------------------------------------------
// <copyright file="FileTreeView.cs" company="Kanazawa-soft-design,LLC.">
//     Copylight(C) 2018 Kanazawa-soft-design,LLC.
// </copyright>
// <author>金澤 宣明</author>
// <summary></summary>
//-----------------------------------------------------------------------

using System;
using System.Windows.Forms;
using System.IO;

namespace WinAppCS001
{
	public partial class MainForm
	{
		// -----------------------------------------------------------------------
		/// <summary>
		/// ディレクトリツリーの初期処理.
		/// </summary>
		private void FileView_Init()
		{
			//! コンピュータ名をディレクトリツリーのルートに登録する.
			var root = AddRootNode();

			//! ディレクトリツリーにドライブ名を登録する.
			AddDriveName( root );

			//! ルートノードを展開して表示する.
			root.Expand();
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// コンピュータ名をツリーのルートノードにセットする.
		/// </summary>
		/// <returns>
		/// 生成したルートノード.
		/// </returns>
		private TreeNode AddRootNode()
		{
			//! コンピュータ名をディレクトリツリーのルートに登録する.
			var root = new TreeNode( Environment.MachineName );

			//! コンピュータ名をツリーのルートノードにセットする.
			FileTree.Nodes.Add( root );

			return root;
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// ディレクトリツリーにドライブ名を登録する.
		/// </summary>
		/// <param name="root">
		/// ディレクトリツリーのルートノード.
		/// </param>
		private void AddDriveName( TreeNode root )
		{
			//! 論理ドライブ名をすべて取得する.
			string[] stDrives = Directory.GetLogicalDrives();

			//! 取得した論理ドライブ名をすべて表示する.
			foreach( var drive in stDrives ){

				//! ドライブ名をの最後にあるセパレータ("\")を取り除く.
				var drvname = drive.Substring( 0, drive.Length - 1 );

				//! ドライブ名をディレクトリツリーに登録する.
				var drvnode = new TreeNode( drvname );
				root.Nodes.Add( drvnode );

				//! ドライブ名直下のディレクトリが存在するかを調べる.
				AddDirectory( drvnode, drive, true );
			}
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// ディレクトリツリーにディレクトリ名を登録する.
		/// </summary>
		/// <param name="parent">
		/// ディレクトリ名を登録するノード.
		/// </param>
		/// <param name="path">
		/// 該当ノードに対応した絶対パスのディレクトリ名.
		/// </param>
		/// <param name="isDummy">
		/// true = サブディレクトリが存在するかどうかだけ調べる.
		/// </param>
		private void AddDirectory( TreeNode parent, string path, bool isDummy )
		{
			path += "\\";

			try {
				//! サブディレクトリを列挙する.
				foreach( var dir in Directory.GetDirectories( path ) ){

					//! ディレクトリの属性を取得する.
					var di = new DirectoryInfo( dir );

					//! 隠しディレクトリはツリーに含まない.
					if( (di.Attributes & FileAttributes.Hidden) != FileAttributes.Hidden ){

						//! ディレクトリをツリーに追加する.
						var subDir = new TreeNode( di.Name );
						parent.Nodes.Add( subDir );

						//! サブディレクトリの存在チェックの場合は、1件検出した時点で処理を終了する.
						if( isDummy ){
							return;
						}
						//! サブディレクトリの存在チェックを行う.
						string subPath = path + subDir.Text;
						AddDirectory( subDir, subPath, true );
					}
				}
			}catch( Exception ){
			}
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// ディレクトリツリーのディレクトリが展開されるときのイベント処理.
		/// </summary>
		/// <param name="sender">
		/// イベントを発生させたオブジェクト.
		/// </param>
		/// <param name="e">
		/// イベント･パラメータ.
		/// </param>
		private void FileTree_BeforeExpand( object sender, TreeViewCancelEventArgs e )
		{
			var node = e.Node;
			if( node == null ){
				return;
			}
			//! 展開されるディレクトリのサブディレクトリを一旦クリアする.
			node.Nodes.Clear();

			//! 展開されるノードに対応したディレクトリの絶対パスを取得する.
			var path = GetAbsolutePath( node );

			if( path.Length > 0 ){
				//! 展開されるディレクトリのサブディレクトリを表示する.
				AddDirectory( node, path, false );
			}else{
				//! 絶対パス名が無い場合はルートの展開なのでドライブ名を表示する.
				AddDriveName( node );
			}
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// ディレクトリツリーのディレクトリの選択が変更されるときのイベント処理.
		/// </summary>
		/// <param name="sender">
		/// イベントを発生させたオブジェクト.
		/// </param>
		/// <param name="e">
		/// イベント･パラメータ.
		/// </param>
		private void FileTree_BeforeSelect( object sender, TreeViewCancelEventArgs e )
		{
			var node = e.Node;
			if( node == null ){
				return;
			}
			//! ファイル一覧を更新する.
			UpdateFileList( GetAbsolutePath( node ) );
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// 指定したディレクトリを展開表示する.
		/// </summary>
		/// <param name="path">
		/// 展開するディレクトリの絶対パス.
		/// </param>
		private void ExpandDirectory( string path )
		{
			//! ディレクトリツリーの該当ディレクトリを展開する.
			ExpandAndSelectDirectory( path );

			//! ファイル一覧を更新する.
			UpdateFileList( path );
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// 指定されたノードに対応した絶対パスを取得する.
		/// </summary>
		/// <param name="node">
		/// 絶対パスを取得するノード.
		/// </param>
		/// <returns>
		/// 指定されたノードに対応した絶対パス.
		/// </returns>
		private string GetAbsolutePath( TreeNode node )
		{
			string path = string.Empty;

			var parent = node.Parent;
			if( parent != null ){
				//! 親ディレクトリの絶対パスを取得する.
				path += GetAbsolutePath( parent );

				//! 親ディレクトリがある場合は連結する為にセパレータ("\")を付ける.
				if( path.Length > 0 ){
					path += "\\";
				}
				//! 親ディレクトリとファイル名を連結して絶対パス名にする.
				path += node.Text;
			}else{
				;
			}
			return path;
		}

		// -----------------------------------------------------------------------
		/// <summary>
		/// 指定されたディレクトリのノードを選択状態にする.
		/// </summary>
		/// <param name="path">
		/// 選択するディレクトリの絶対パス名.
		/// </param>
		private void ExpandAndSelectDirectory( string path )
		{
			TreeNode node = FileTree.Nodes[0];

			//! IndexOf()検索で最後まで検索するようにパスの最後に検索文字を付加する.
			string wkPath = path + "\\";

			int nextPos;
			int nowPos = 0;
			while( node != null ){

				//! ディレクトリ名を取り出す.
				nextPos = wkPath.IndexOf( '\\', nowPos );
				if( nextPos < 0 ){
					break;
				}

				//! 子ノードが無い場合は処理を終了する.
				if( node.Nodes.Count == 0 ){
					node = null;
					break;
				}

				//! "\"が連続している場合は読み飛ばす.
				if( nextPos == nowPos ){
					nowPos = nextPos + 1;
					continue;
				}

				//! ディレクトリ名を絶対パスから取り出す.
				string dirName = wkPath.Substring( nowPos, (nextPos - nowPos) );
				nowPos = nextPos + 1;

				foreach( TreeNode wkNode in node.Nodes ){
					//! 同じディレクトリ名のノードを見つけたらループを抜ける.
					if( dirName.Equals( wkNode.Text, StringComparison.OrdinalIgnoreCase ) ){
						if( !wkNode.IsExpanded ){
							wkNode.Expand();
						}
						node = wkNode;
						break;
					}
				}
			}
			//! ノードを選択状態にする.
			if( node != null ){
				FileTree.SelectedNode = node;
				FileTree.Focus();
			}
		}
	}
}
